"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var postcss_1 = __importStar(require("postcss"));
var rtlcss_1 = __importDefault(require("rtlcss"));
var _types_1 = require("../@types");
var _constants_1 = require("../constants");
var store_1 = require("../data/store");
var selectors_1 = require("../utilities/selectors");
var shorthands_1 = require("../utilities/shorthands");
var containers_1 = require("../utilities/containers");
exports.parseDeclarations = function (rule) {
    var _a = store_1.store.options, mode = _a.mode, ltrPrefix = _a.ltrPrefix, rtlPrefix = _a.rtlPrefix, bothPrefix = _a.bothPrefix, source = _a.source, processUrls = _a.processUrls, useCalc = _a.useCalc, stringMap = _a.stringMap, autoRename = _a.autoRename, greedy = _a.greedy;
    var deleteDeclarations = [];
    var ruleFlipped = rule.clone().removeAll();
    var ruleFlippedSecond = ruleFlipped.clone();
    var ruleBoth = ruleFlipped.clone();
    var declarationHashMap = Array.prototype.reduce.call(rule.nodes, function (obj, node) {
        if (node.type === _constants_1.DECLARATION_TYPE) {
            var decl = node;
            obj[decl.prop] = decl.value.trim();
        }
        return obj;
    }, {});
    var declarationsProps = [];
    var simetricRules = false;
    containers_1.walkContainer(rule, [_constants_1.DECLARATION_TYPE], true, function (node) {
        var decl = node;
        var declString = decl.toString() + ";";
        var declFlippedString = rtlcss_1.default.process(declString, {
            processUrls: processUrls,
            useCalc: useCalc,
            stringMap: stringMap,
            autoRename: autoRename !== _types_1.Autorename.disabled,
            autoRenameStrict: autoRename === _types_1.Autorename.strict,
            greedy: greedy
        });
        var root = postcss_1.default.parse(declFlippedString);
        var declFlipped = root.first;
        declFlipped.raws = decl.raws;
        var declProp = decl.prop.trim();
        var declPropUnprefixed = postcss_1.vendor.unprefixed(declProp);
        var declValue = decl.value.trim();
        var isAnimation = declPropUnprefixed === _constants_1.ANIMATION_PROP || declPropUnprefixed === _constants_1.ANIMATION_NAME_PROP;
        var declFlippedProp = declFlipped.prop.trim();
        var declFlippedValue = declFlipped.value.trim();
        var overridenBy = shorthands_1.shorthands[declPropUnprefixed];
        var hasBeenOverriden = overridenBy
            ? overridenBy.some(function (d) { return declarationsProps.indexOf(d) >= 0; })
            : false;
        if (!hasBeenOverriden &&
            declProp === declFlippedProp &&
            declValue === declFlippedValue &&
            (!isAnimation ||
                (isAnimation &&
                    (store_1.store.keyframes.length === 0 ||
                        !store_1.store.keyframesRegExp.test(declValue))))) {
            return;
        }
        if (isAnimation) {
            var declValue_1 = decl.value.replace(store_1.store.keyframesRegExp, function (_match, before, animation, after) {
                return before + store_1.store.keyframesStringMap[animation].name + after;
            });
            var declValueFlipped = decl.value.replace(store_1.store.keyframesRegExp, function (_match, before, animation, after) {
                return before + store_1.store.keyframesStringMap[animation].nameFlipped + after;
            });
            var declCloneFlipped = decl.clone();
            if (mode === _types_1.Mode.combined) {
                var declClone = decl.clone();
                declClone.value = declValue_1;
                declCloneFlipped.value = declValueFlipped;
                ruleFlipped.append(declClone);
                ruleFlippedSecond.append(declCloneFlipped);
                deleteDeclarations.push(decl);
            }
            else {
                var declCloneFlipped_1 = decl.clone();
                decl.value = declValue_1;
                declCloneFlipped_1.value = declValueFlipped;
                ruleFlipped.append(declCloneFlipped_1);
            }
        }
        else {
            if (hasBeenOverriden &&
                declProp === declFlippedProp &&
                declValue === declFlippedValue) {
                var declClone = decl.clone();
                ruleBoth.append(declClone);
                deleteDeclarations.push(decl);
                return;
            }
            else if (declarationHashMap[declFlipped.prop] === declFlippedValue) {
                simetricRules = true;
                return;
            }
            if (mode === _types_1.Mode.combined) {
                var declClone = decl.clone();
                ruleFlipped.append(declClone);
                ruleFlippedSecond.append(declFlipped);
                deleteDeclarations.push(decl);
            }
            else {
                if (_constants_1.FLIP_PROPERTY_REGEXP.test(decl.prop) && !declarationHashMap[declFlipped.prop]) {
                    var declClone = decl.clone();
                    declClone.value = 'unset';
                    ruleFlipped.append(declClone);
                }
                ruleFlipped.append(declFlipped);
            }
            declarationsProps.push(declPropUnprefixed);
        }
    });
    if (deleteDeclarations.length) {
        deleteDeclarations.forEach(function (decl) { return decl.remove(); });
    }
    if (ruleFlipped.nodes.length || ruleFlippedSecond.nodes.length) {
        if (mode === _types_1.Mode.combined) {
            selectors_1.addSelectorPrefixes(ruleFlipped, source === _types_1.Source.ltr ? ltrPrefix : rtlPrefix);
            selectors_1.addSelectorPrefixes(ruleFlippedSecond, source === _types_1.Source.rtl ? ltrPrefix : rtlPrefix);
        }
        else {
            selectors_1.addSelectorPrefixes(ruleFlipped, source === _types_1.Source.ltr ? rtlPrefix : ltrPrefix);
            selectors_1.addSelectorPrefixes(ruleFlippedSecond, source === _types_1.Source.rtl ? rtlPrefix : ltrPrefix);
        }
        selectors_1.addSelectorPrefixes(ruleBoth, bothPrefix);
        store_1.store.rules.push({
            rule: rule,
            ruleLTR: ruleFlipped,
            ruleRTL: ruleFlippedSecond,
            ruleBoth: ruleBoth
        });
    }
    else if (!simetricRules) {
        store_1.store.rulesAutoRename.push(rule);
    }
};
