"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _types_1 = require("../@types");
var _constants_1 = require("../constants");
var store_1 = require("../data/store");
exports.cleanRuleRawsBefore = function (node) {
    if (node && node.type === _constants_1.RULE_TYPE) {
        node.raws.before = '\n\n' + node.raws.before.replace(/\n/g, '');
    }
};
exports.cleanRules = function () {
    var rules = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        rules[_i] = arguments[_i];
    }
    rules.forEach(function (rule) {
        var prev = rule.prev();
        if (prev && prev.type !== _constants_1.COMMENT_TYPE) {
            exports.cleanRuleRawsBefore(rule);
        }
        rule.walk(function (node) {
            if (node.type === _constants_1.DECLARATION_TYPE) {
                if (node.raws && node.raws.value && _constants_1.RTL_COMMENT_REGEXP.test(node.raws.value.raw)) {
                    delete node.raws.value;
                    node.value = node.value.trim();
                }
            }
        });
    });
};
exports.appendRules = function () {
    var rules = store_1.store.rules;
    rules.forEach(function (_a) {
        var rule = _a.rule, ruleLTR = _a.ruleLTR, ruleRTL = _a.ruleRTL, ruleBoth = _a.ruleBoth;
        ruleBoth && ruleBoth.nodes.length && rule.after(ruleBoth);
        ruleRTL && ruleRTL.nodes.length && rule.after(ruleRTL);
        ruleLTR && ruleLTR.nodes.length && rule.after(ruleLTR);
        if (rule.nodes.length === 0) {
            rule.remove();
        }
        exports.cleanRules(rule, ruleLTR, ruleRTL, ruleBoth);
    });
};
exports.appendKeyFrames = function () {
    var keyframes = store_1.store.keyframes;
    keyframes.forEach(function (_a) {
        var atRule = _a.atRule, atRuleFlipped = _a.atRuleFlipped;
        atRule.after(atRuleFlipped);
        exports.cleanRules(atRule, atRuleFlipped);
    });
};
exports.appendAutorenameRules = function () {
    var replaceHash = store_1.store.options.stringMap.reduce(function (hash, map) {
        var search = typeof map.search === 'string' ? [map.search] : map.search;
        var replace = typeof map.replace === 'string' ? [map.replace] : map.replace;
        search.forEach(function (s, index) {
            hash[s] = replace[index];
            hash[replace[index]] = s;
        });
        return hash;
    }, {});
    var replaces = Object.keys(replaceHash).join('|');
    var replaceRegExp = store_1.store.options.greedy
        ? new RegExp("(" + replaces + ")", 'g')
        : new RegExp("\\b(" + replaces + ")\\b", 'g');
    var rulesHash = {};
    var rulesToProcess = [];
    store_1.store.rulesAutoRename.forEach(function (rule) {
        var process = false;
        rule.selectors.forEach(function (selector) {
            if (replaceRegExp.test(selector)) {
                rulesHash[selector] = true;
                process = true;
            }
            replaceRegExp.lastIndex = 0;
        });
        if (process) {
            rulesToProcess.push(rule);
        }
    });
    rulesToProcess.forEach(function (rule) {
        rule.selectors = rule.selectors.map(function (selector) {
            var flip = selector.replace(replaceRegExp, function (match, group) { return replaceHash[group]; });
            if (store_1.store.options.autoRename === _types_1.Autorename.flexible ||
                (store_1.store.options.autoRename === _types_1.Autorename.strict && rulesHash[flip])) {
                return flip;
            }
            return selector;
        });
    });
};
